<?php
/**
 * 管理画面ページを提供するクラス（実行状況表示機能付き）
 */
class WP_Revision_Cleaner_Admin {

    /**
     * 初期設定
     */
    public function setup() {
        // 管理メニューを追加
        add_action('admin_menu', array($this, 'add_admin_menu'));
        
        // スクリプトとスタイルを登録
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_scripts'));
        
        // プラグイン有効化後の通知
        add_action('admin_notices', array($this, 'activation_notice'));
        
        // バックグラウンド処理のアクションを追加
        add_action('wp_revision_cleaner_background_process', array($this, 'run_background_process'));
    }
    
    /**
     * バックグラウンド処理を実行
     */
    public function run_background_process() {
        // クリーナークラスを取得
        global $wp_revision_cleaner;
        if ($wp_revision_cleaner) {
            // バックグラウンドモードで実行
            $wp_revision_cleaner->cleanup_old_revisions(true);
        }
    }
    
    /**
     * 管理メニューを追加
     */
    public function add_admin_menu() {
        add_management_page(
            __('リビジョン整理ツール', 'wp-revision-cleaner'), 
            __('リビジョン整理', 'wp-revision-cleaner'), 
            'manage_options', 
            'revision-cleanup', 
            array($this, 'render_admin_page')
        );
    }
    
    /**
     * 管理画面スクリプトとスタイルの読み込み
     */
    public function enqueue_admin_scripts($hook) {
        // リビジョン整理ページのみにスクリプトを読み込む
        if ($hook !== 'tools_page_revision-cleanup') {
            return;
        }
        
        // スタイルの登録と読み込み
        wp_enqueue_style(
            'wp-revision-cleaner-admin',
            WP_REVISION_CLEANER_URL . 'admin/css/admin-style.css',
            array(),
            WP_REVISION_CLEANER_VERSION
        );
        
        // スクリプトの登録と読み込み
        wp_enqueue_script(
            'wp-revision-cleaner-admin',
            WP_REVISION_CLEANER_URL . 'admin/js/admin-script.js',
            array('jquery'),
            WP_REVISION_CLEANER_VERSION,
            true
        );
        
        // ローカライゼーションデータを追加
        wp_localize_script(
            'wp-revision-cleaner-admin',
            'wpRevisionCleaner',
            array(
                'ajaxurl' => admin_url('admin-ajax.php'),
                'nonce' => wp_create_nonce('wp_revision_cleaner_nonce'),
                'confirmRun' => __('リビジョンの整理を実行しますか？', 'wp-revision-cleaner'),
                'showDetails' => __('詳細を表示', 'wp-revision-cleaner'),
                'hideDetails' => __('詳細を隠す', 'wp-revision-cleaner'),
                'refreshInterval' => 3000, // ステータス更新間隔（ミリ秒）
                'strings' => array(
                    'processingStatus' => __('処理状況', 'wp-revision-cleaner'),
                    'startTime' => __('開始時間', 'wp-revision-cleaner'),
                    'currentStatus' => __('現在の状態', 'wp-revision-cleaner'),
                    'progress' => __('進捗', 'wp-revision-cleaner'),
                    'processed' => __('処理済み', 'wp-revision-cleaner'),
                    'deleted' => __('削除済み', 'wp-revision-cleaner'),
                    'preserved' => __('保持済み', 'wp-revision-cleaner'),
                    'recentActivity' => __('最近の処理', 'wp-revision-cleaner'),
                    'action' => __('アクション', 'wp-revision-cleaner'),
                    'title' => __('タイトル', 'wp-revision-cleaner'),
                    'date' => __('日時', 'wp-revision-cleaner'),
                    'noActivity' => __('まだ処理がありません', 'wp-revision-cleaner'),
                    'statusNotRunning' => __('処理は実行されていません', 'wp-revision-cleaner')
                )
            )
        );
    }
    
    /**
     * 有効化通知の表示
     */
    public function activation_notice() {
        // 有効化されたばかりかチェック
        if (get_option('wp_revision_cleaner_activated')) {
            ?>
            <div class="notice notice-success is-dismissible">
                <p><?php _e('WP Revision Cleanerが有効化されました。初期設定では毎日午前3時に古いリビジョンを整理します。詳細設定は<a href="' . admin_url('tools.php?page=revision-cleanup') . '">こちら</a>から確認できます。', 'wp-revision-cleaner'); ?></p>
            </div>
            <?php
            // 通知を一度だけ表示するよう、オプションを削除
            delete_option('wp_revision_cleaner_activated');
        }
    }
    
    /**
     * 管理画面ページのレンダリング
     */
    public function render_admin_page() {
        // 次回の実行時間を取得
        $next_run = wp_next_scheduled('daily_revision_cleanup');
        $logs = get_option(REVISION_CLEANUP_LOG, array());
        
        // バックグラウンド実行中かチェック
        $background_running = isset($_GET['background']) && $_GET['background'] === 'running';
        
        ?>
        <div class="wrap wp-revision-cleaner">
            <h1><?php _e('WordPressリビジョン整理ツール', 'wp-revision-cleaner'); ?></h1>
            
            <?php if ($background_running): ?>
            <!-- 実行状況表示エリア -->
            <div class="card" id="progress-container">
                <h2><?php _e('リビジョン整理の実行状況', 'wp-revision-cleaner'); ?></h2>
                
                <div id="progress-status">
                    <p class="loading"><?php _e('状態を取得中...', 'wp-revision-cleaner'); ?></p>
                </div>
                
                <div id="progress-bar-container" class="hidden">
                    <div class="progress-label">
                        <span id="progress-percentage">0%</span> 完了
                    </div>
                    <div class="progress-bar">
                        <div class="progress-bar-inner" style="width: 0%;"></div>
                    </div>
                </div>
                
                <div id="progress-details" class="hidden">
                    <h3><?php _e('処理状況', 'wp-revision-cleaner'); ?></h3>
                    <table class="widefat">
                        <tr>
                            <th><?php _e('開始時間', 'wp-revision-cleaner'); ?></th>
                            <td id="start-time">-</td>
                        </tr>
                        <tr>
                            <th><?php _e('現在の状態', 'wp-revision-cleaner'); ?></th>
                            <td id="current-action">-</td>
                        </tr>
                        <tr>
                            <th><?php _e('処理済み', 'wp-revision-cleaner'); ?></th>
                            <td id="processed-count">0 / 0</td>
                        </tr>
                        <tr>
                            <th><?php _e('削除済み', 'wp-revision-cleaner'); ?></th>
                            <td id="deleted-count">0</td>
                        </tr>
                        <tr>
                            <th><?php _e('保持済み', 'wp-revision-cleaner'); ?></th>
                            <td id="preserved-count">0</td>
                        </tr>
                    </table>
                </div>
                
                <div id="recent-activity" class="hidden">
                    <h3><?php _e('最近の処理', 'wp-revision-cleaner'); ?></h3>
                    <table class="widefat">
                        <thead>
                            <tr>
                                <th><?php _e('アクション', 'wp-revision-cleaner'); ?></th>
                                <th><?php _e('タイトル', 'wp-revision-cleaner'); ?></th>
                                <th><?php _e('日時', 'wp-revision-cleaner'); ?></th>
                            </tr>
                        </thead>
                        <tbody id="activity-rows">
                            <tr>
                                <td colspan="3"><?php _e('まだ処理がありません', 'wp-revision-cleaner'); ?></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
                
                <p class="submit">
                    <a href="<?php echo admin_url('tools.php?page=revision-cleanup'); ?>" class="button" id="back-button"><?php _e('戻る', 'wp-revision-cleaner'); ?></a>
                    <button type="button" class="button button-primary" id="refresh-status"><?php _e('状態を更新', 'wp-revision-cleaner'); ?></button>
                </p>
            </div>
            <?php else: ?>
            <!-- 通常の管理画面 -->
            <div class="card">
                <h2><?php _e('リビジョン整理の状況', 'wp-revision-cleaner'); ?></h2>
                <p><?php _e('このツールは一週間以上前のリビジョンを整理し、1日あたり最新の1つだけを残します。', 'wp-revision-cleaner'); ?></p>
                
                <?php if ($next_run): ?>
                    <p><?php _e('次回の自動実行予定:', 'wp-revision-cleaner'); ?> <strong><?php echo date_i18n('Y年n月j日 G時i分', $next_run); ?></strong></p>
                <?php else: ?>
                    <p><?php _e('自動実行がスケジュールされていません。プラグインを再有効化してみてください。', 'wp-revision-cleaner'); ?></p>
                <?php endif; ?>
                
                <form method="post" action="<?php echo admin_url('admin-post.php'); ?>">
                    <input type="hidden" name="action" value="run_revision_cleanup">
                    <?php wp_nonce_field('run_revision_cleanup'); ?>
                    
                    <p>
                        <label>
                            <input type="checkbox" name="background_execution" value="1" checked>
                            <?php _e('バックグラウンドで実行（進行状況を表示）', 'wp-revision-cleaner'); ?>
                        </label>
                    </p>
                    
                    <p><button type="submit" class="button button-primary"><?php _e('今すぐ実行', 'wp-revision-cleaner'); ?></button></p>
                </form>
            </div>
            
            <?php if (isset($_GET['run']) && $_GET['run'] === 'success'): ?>
                <div class="notice notice-success is-dismissible">
                    <p><?php 
                        printf(
                            __('リビジョン整理が完了しました。%d件を削除し、%d件の古いリビジョンを保持しました。', 'wp-revision-cleaner'),
                            intval($_GET['deleted']),
                            intval($_GET['preserved'])
                        ); 
                    ?></p>
                </div>
            <?php endif; ?>
            
            <div class="card">
                <h2><?php _e('実行履歴', 'wp-revision-cleaner'); ?></h2>
                
                <?php if (empty($logs)): ?>
                    <p><?php _e('まだ実行履歴がありません。', 'wp-revision-cleaner'); ?></p>
                <?php else: ?>
                    <table class="widefat striped">
                        <thead>
                            <tr>
                                <th><?php _e('実行日時', 'wp-revision-cleaner'); ?></th>
                                <th><?php _e('結果', 'wp-revision-cleaner'); ?></th>
                                <th><?php _e('詳細', 'wp-revision-cleaner'); ?></th>
                            </tr>
                        </thead>
                        <tbody>
                            <?php foreach ($logs as $log): ?>
                                <tr>
                                    <td><?php echo date_i18n('Y年n月j日 G時i分s秒', strtotime($log['time'])); ?></td>
                                    <td class="<?php echo $log['status'] === 'error' ? 'error' : ''; ?>">
                                        <?php echo esc_html($log['message']); ?>
                                    </td>
                                    <td>
                                        <?php if (!empty($log['details'])): ?>
                                            <button type="button" class="button toggle-details"><?php _e('詳細を表示', 'wp-revision-cleaner'); ?></button>
                                            <div class="hidden details-content">
                                                <h4><?php _e('削除されたリビジョン:', 'wp-revision-cleaner'); ?></h4>
                                                <ul>
                                                    <?php foreach (array_slice($log['details'], 0, 10) as $detail): ?>
                                                        <li><?php 
                                                            printf(
                                                                __('「%s」(ID:%d)のリビジョン - %s', 'wp-revision-cleaner'),
                                                                esc_html($detail['parent_title']),
                                                                $detail['parent_id'],
                                                                date_i18n('Y年n月j日 G時i分', strtotime($detail['date']))
                                                            ); 
                                                        ?></li>
                                                    <?php endforeach; ?>
                                                    
                                                    <?php if (count($log['details']) > 10): ?>
                                                        <li><?php 
                                                            printf(
                                                                __('他 %d 件...', 'wp-revision-cleaner'),
                                                                count($log['details']) - 10
                                                            ); 
                                                        ?></li>
                                                    <?php endif; ?>
                                                </ul>
                                            </div>
                                        <?php endif; ?>
                                    </td>
                                </tr>
                            <?php endforeach; ?>
                        </tbody>
                    </table>
                <?php endif; ?>
            </div>
            <?php endif; ?>
        </div>
        <?php
    }
}