/**
 * WP Revision Cleaner 管理画面スクリプト（実行状況表示機能付き）
 */
(function($) {
    'use strict';
    
    // 自動更新タイマーID
    var statusTimer = null;
    
    $(document).ready(function() {
        // 詳細表示ボタンのクリックイベント
        $('.wp-revision-cleaner .toggle-details').on('click', function() {
            var $content = $(this).next('.details-content');
            $content.toggleClass('hidden');
            
            if ($content.hasClass('hidden')) {
                $(this).text(wpRevisionCleaner.showDetails);
            } else {
                $(this).text(wpRevisionCleaner.hideDetails);
            }
        });
        
        // 実行ボタン押下時の確認ダイアログ
        $('.wp-revision-cleaner form').on('submit', function(e) {
            if (!confirm(wpRevisionCleaner.confirmRun)) {
                e.preventDefault();
                return false;
            }
        });
        
        // 進行状況画面の処理
        if ($('#progress-container').length > 0) {
            // 初回読み込み
            refreshStatus();
            
            // 定期的に更新
            statusTimer = setInterval(refreshStatus, wpRevisionCleaner.refreshInterval);
            
            // 手動更新ボタン
            $('#refresh-status').on('click', function() {
                refreshStatus();
            });
        }
    });
    
    /**
     * 実行状況を更新する
     */
    function refreshStatus() {
        $.ajax({
            url: wpRevisionCleaner.ajaxurl,
            type: 'POST',
            data: {
                action: 'get_revision_cleanup_status',
                nonce: wpRevisionCleaner.nonce
            },
            success: function(response) {
                if (response.success) {
                    updateStatusDisplay(response.data);
                } else {
                    $('#progress-status').html('<p class="error">' + response.data + '</p>');
                }
            },
            error: function() {
                $('#progress-status').html('<p class="error">ステータスの取得に失敗しました。</p>');
            }
        });
    }
    
    /**
     * 状態表示を更新する
     */
    function updateStatusDisplay(status) {
        // 実行中かどうかで表示を切り替え
        if (status.running) {
            $('#progress-status').html('<p class="running">' + status.current_action + '</p>');
            $('#progress-bar-container').removeClass('hidden');
            $('#progress-details').removeClass('hidden');
            $('#recent-activity').removeClass('hidden');
            
            // 進捗バーの更新
            var percent = 0;
            if (status.total > 0) {
                percent = Math.round((status.processed / status.total) * 100);
            }
            $('#progress-percentage').text(percent + '%');
            $('.progress-bar-inner').css('width', percent + '%');
            
            // 詳細情報の更新
            $('#start-time').text(formatDateTime(status.start_time));
            $('#current-action').text(status.current_action);
            $('#processed-count').text(status.processed + ' / ' + status.total);
            $('#deleted-count').text(status.deleted);
            $('#preserved-count').text(status.preserved);
            
            // 最近の処理を更新
            updateRecentActivity(status.recent_items);
        } else {
            // 実行終了または未実行
            if (status.start_time > 0) {
                // 実行完了
                $('#progress-status').html('<p class="complete">' + status.current_action + '</p>');
                
                // 処理完了の詳細を表示
                $('#progress-bar-container').removeClass('hidden');
                $('#progress-details').removeClass('hidden');
                $('#recent-activity').removeClass('hidden');
                
                // 進捗バーを100%に
                $('#progress-percentage').text('100%');
                $('.progress-bar-inner').css('width', '100%');
                
                // 詳細情報の更新
                $('#start-time').text(formatDateTime(status.start_time));
                $('#current-action').text(status.current_action);
                $('#processed-count').text(status.processed + ' / ' + status.total);
                $('#deleted-count').text(status.deleted);
                $('#preserved-count').text(status.preserved);
                
                // 最近の処理を更新
                updateRecentActivity(status.recent_items);
                
                // 更新を停止
                if (statusTimer) {
                    clearInterval(statusTimer);
                    statusTimer = null;
                }
            } else {
                // 未実行
                $('#progress-status').html('<p>' + wpRevisionCleaner.strings.statusNotRunning + '</p>');
            }
        }
    }
    
    /**
     * 最近の処理リストを更新
     */
    function updateRecentActivity(items) {
        var $container = $('#activity-rows');
        $container.empty();
        
        if (!items || items.length === 0) {
            $container.append('<tr><td colspan="3">' + wpRevisionCleaner.strings.noActivity + '</td></tr>');
            return;
        }
        
        // 最新の項目を上に表示
        items.reverse();
        
        $.each(items, function(i, item) {
            var actionClass = item.action === '削除' ? 'delete-action' : 'preserve-action';
            
            $container.append(
                '<tr>' +
                    '<td class="' + actionClass + '">' + item.action + '</td>' +
                    '<td>' + item.title + '</td>' +
                    '<td>' + formatDateTime(new Date(item.date).getTime() / 1000) + '</td>' +
                '</tr>'
            );
        });
    }
    
    /**
     * UNIXタイムスタンプを整形する
     */
    function formatDateTime(timestamp) {
        if (!timestamp) return '-';
        
        var date = new Date(timestamp * 1000);
        
        var year = date.getFullYear();
        var month = date.getMonth() + 1;
        var day = date.getDate();
        var hours = date.getHours();
        var minutes = date.getMinutes();
        var seconds = date.getSeconds();
        
        // ゼロ埋め
        month = (month < 10) ? '0' + month : month;
        day = (day < 10) ? '0' + day : day;
        hours = (hours < 10) ? '0' + hours : hours;
        minutes = (minutes < 10) ? '0' + minutes : minutes;
        seconds = (seconds < 10) ? '0' + seconds : seconds;
        
        return year + '年' + month + '月' + day + '日 ' + hours + ':' + minutes + ':' + seconds;
    }
    
})(jQuery);