<?php
/**
 * リビジョン整理の中核機能を提供するクラス（実行状況表示機能付き）
 */
class WP_Revision_Cleaner {

    /**
     * 処理状況を保存するオプション名
     */
    const PROCESS_STATUS_OPTION = 'wp_revision_cleaner_status';

    /**
     * 初期設定
     */
    public function setup() {
        // 自動実行のアクションフックを追加
        add_action('daily_revision_cleanup', array($this, 'cleanup_old_revisions'));
        
        // 手動実行用のアクションを追加
        add_action('admin_post_run_revision_cleanup', array($this, 'handle_manual_run'));
        
        // 状態確認用のAJAXアクションを追加
        add_action('wp_ajax_get_revision_cleanup_status', array($this, 'get_status'));
        
        // 状態リセット用のアクションを追加
        add_action('wp_ajax_reset_revision_cleanup_status', array($this, 'reset_status'));
    }

    /**
     * 実行状況を取得するAJAXハンドラー
     */
    public function get_status() {
        // 管理者権限チェック
        if (!current_user_can('manage_options')) {
            wp_send_json_error('権限がありません');
        }
        
        $status = get_option(self::PROCESS_STATUS_OPTION, array(
            'running' => false,
            'total' => 0,
            'processed' => 0,
            'deleted' => 0,
            'preserved' => 0,
            'start_time' => 0,
            'last_update' => 0,
            'current_action' => '',
            'recent_items' => array()
        ));
        
        // 5分以上更新がない場合は処理が停止したとみなす
        if ($status['running'] && time() - $status['last_update'] > 300) {
            $status['running'] = false;
            $status['current_action'] = '処理が中断されました';
            update_option(self::PROCESS_STATUS_OPTION, $status);
        }
        
        wp_send_json_success($status);
    }
    
    /**
     * 実行状況をリセットするAJAXハンドラー
     */
    public function reset_status() {
        // 管理者権限チェック
        if (!current_user_can('manage_options')) {
            wp_send_json_error('権限がありません');
        }
        
        $default_status = array(
            'running' => false,
            'total' => 0,
            'processed' => 0,
            'deleted' => 0,
            'preserved' => 0,
            'start_time' => 0,
            'last_update' => 0,
            'current_action' => '',
            'recent_items' => array()
        );
        
        update_option(self::PROCESS_STATUS_OPTION, $default_status);
        wp_send_json_success('ステータスをリセットしました');
    }
    
    /**
     * 実行状況を更新する
     */
    private function update_status($data) {
        $status = get_option(self::PROCESS_STATUS_OPTION, array(
            'running' => false,
            'total' => 0,
            'processed' => 0,
            'deleted' => 0,
            'preserved' => 0,
            'start_time' => 0,
            'last_update' => 0,
            'current_action' => '',
            'recent_items' => array()
        ));
        
        // データをマージ
        $status = array_merge($status, $data);
        $status['last_update'] = time();
        
        // 最近の項目を制限
        if (isset($status['recent_items']) && count($status['recent_items']) > 10) {
            $status['recent_items'] = array_slice($status['recent_items'], -10);
        }
        
        update_option(self::PROCESS_STATUS_OPTION, $status);
        
        // 大きなサイトではメモリを解放
        if (function_exists('wp_cache_flush')) {
            wp_cache_flush();
        }
    }
    
    /**
     * 手動実行のハンドラー
     */
    public function handle_manual_run() {
        // セキュリティチェック
        check_admin_referer('run_revision_cleanup');
        
        // 管理者権限チェック
        if (!current_user_can('manage_options')) {
            wp_die(__('この操作を実行する権限がありません。', 'wp-revision-cleaner'));
        }
        
        // バックグラウンド実行フラグがある場合は非同期実行を開始
        if (isset($_POST['background_execution']) && $_POST['background_execution'] == '1') {
            // ステータスを初期化
            $this->update_status(array(
                'running' => true,
                'total' => 0,
                'processed' => 0,
                'deleted' => 0,
                'preserved' => 0,
                'start_time' => time(),
                'current_action' => '処理を開始しています...'
            ));
            
            // 非同期処理をスケジュール
            wp_schedule_single_event(time(), 'wp_revision_cleaner_background_process');
            
            // 状況確認ページにリダイレクト
            wp_redirect(admin_url('tools.php?page=revision-cleanup&background=running'));
            exit;
        }
        
        // 通常の同期実行
        $result = $this->cleanup_old_revisions();
        
        // 結果ページにリダイレクト
        wp_redirect(admin_url('tools.php?page=revision-cleanup&run=success&deleted=' . $result['deleted'] . '&preserved=' . $result['preserved']));
        exit;
    }

    /**
     * 古いリビジョンを整理する
     * 
     * @param bool $background バックグラウンド実行モードなら true
     * @return array 実行結果の統計
     */
    public function cleanup_old_revisions($background = false) {
        global $wpdb;
        
        // バックグラウンド実行の場合は状態を更新
        if ($background) {
            $this->update_status(array(
                'running' => true,
                'current_action' => 'データベースからリビジョンを検索中...'
            ));
        }
        
        // 7日前の日付を取得
        $date_before = date('Y-m-d', strtotime('-7 days'));
        
        // リビジョンを日付でグループ化して取得（日付ごとに最新順）
        $revisions_by_date = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT 
                    DATE(post_date) as revision_date,
                    ID,
                    post_parent,
                    post_date,
                    post_title
                FROM 
                    {$wpdb->posts}
                WHERE 
                    post_type = 'revision'
                    AND DATE(post_date) < %s
                ORDER BY 
                    post_parent ASC,
                    revision_date ASC,
                    post_date DESC",
                $date_before
            )
        );
        
        if ($wpdb->last_error) {
            $this->record_cleanup_log("SQLエラーが発生しました: " . $wpdb->last_error, 'error');
            
            if ($background) {
                $this->update_status(array(
                    'running' => false,
                    'current_action' => 'エラーが発生しました: ' . $wpdb->last_error
                ));
            }
            
            return array(
                'deleted' => 0,
                'preserved' => 0,
                'details' => array()
            );
        }
        
        // 合計アイテム数を記録
        $total_revisions = count($revisions_by_date);
        
        if ($background) {
            $this->update_status(array(
                'total' => $total_revisions,
                'current_action' => "{$total_revisions}件のリビジョンを処理中..."
            ));
        }
        
        // 日付と投稿ごとにリビジョンをグループ化
        $grouped_revisions = array();
        foreach ($revisions_by_date as $revision) {
            $key = $revision->post_parent . '_' . $revision->revision_date;
            if (!isset($grouped_revisions[$key])) {
                $grouped_revisions[$key] = array();
            }
            $grouped_revisions[$key][] = $revision;
        }
        
        // 各グループから最初の1つ（最新）を残して削除
        $deleted_count = 0;
        $preserved_count = 0;
        $deleted_details = array();
        $processed_count = 0;
        
        foreach ($grouped_revisions as $key => $revisions) {
            // 処理状況を更新（10グループごと）
            if ($background && $processed_count % 10 === 0) {
                $progress_percent = ($processed_count / count($grouped_revisions)) * 100;
                $this->update_status(array(
                    'processed' => $processed_count,
                    'deleted' => $deleted_count,
                    'preserved' => $preserved_count,
                    'current_action' => sprintf('処理中... %.1f%% 完了', $progress_percent)
                ));
            }
            
            // 最初の要素（最新のリビジョン）をスキップ
            $preserved = array_shift($revisions);
            $preserved_count++;
            
            if ($background) {
                $parent_post = get_post($preserved->post_parent);
                $parent_title = $parent_post ? $parent_post->post_title : '不明な投稿';
                
                // 保持したリビジョンを記録
                $this->update_status(array(
                    'recent_items' => array(
                        array(
                            'action' => '保持',
                            'title' => $parent_title,
                            'date' => $preserved->post_date
                        )
                    )
                ));
            }
            
            // 残りを削除
            foreach ($revisions as $revision) {
                $parent_post = get_post($revision->post_parent);
                $parent_title = $parent_post ? $parent_post->post_title : '不明な投稿';
                
                // 状態を更新
                if ($background) {
                    $current_status = get_option(self::PROCESS_STATUS_OPTION);
                    $current_status['recent_items'][] = array(
                        'action' => '削除',
                        'title' => $parent_title,
                        'id' => $revision->ID,
                        'date' => $revision->post_date
                    );
                    $this->update_status($current_status);
                }
                
                $result = wp_delete_post_revision($revision->ID);
                if ($result) {
                    $deleted_count++;
                    // 削除されたリビジョンの情報を記録
                    $deleted_details[] = array(
                        'id' => $revision->ID,
                        'parent_id' => $revision->post_parent,
                        'parent_title' => $parent_title,
                        'date' => $revision->post_date,
                    );
                }
            }
            
            $processed_count++;
        }
        
        // 実行結果を記録
        $log_message = "実行結果: {$deleted_count}件のリビジョンを削除しました。{$preserved_count}件の古いリビジョンを保持しました。";
        $this->record_cleanup_log($log_message, 'success', $deleted_details);
        
        // バックグラウンド実行の場合は完了状態を更新
        if ($background) {
            $this->update_status(array(
                'running' => false,
                'processed' => $processed_count,
                'deleted' => $deleted_count,
                'preserved' => $preserved_count,
                'current_action' => "完了: {$deleted_count}件のリビジョンを削除しました"
            ));
        }
        
        return array(
            'deleted' => $deleted_count,
            'preserved' => $preserved_count,
            'details' => $deleted_details
        );
    }
    
    /**
     * 実行ログを記録する
     * 
     * @param string $message ログメッセージ
     * @param string $status ステータス (success, error, info)
     * @param array $details 詳細情報
     */
    public function record_cleanup_log($message, $status = 'info', $details = array()) {
        $current_logs = get_option(REVISION_CLEANUP_LOG, array());
        
        // ログが大きくなりすぎないよう、最大50件に制限
        if (count($current_logs) >= 50) {
            array_pop($current_logs);
        }
        
        // 新しいログを先頭に追加
        array_unshift($current_logs, array(
            'time' => current_time('mysql'),
            'message' => $message,
            'status' => $status,
            'details' => $details
        ));
        
        update_option(REVISION_CLEANUP_LOG, $current_logs);
    }
}